package main

import (
	"flag"
	"git.randomchars.net/freenitori/log"
	"github.com/BurntSushi/toml"
	"os"
	"time"
)

var config configPayload
var configPath string
var defaultConfig = configPayload{
	Prefix:           "!",
	Token:            "TOKEN",
	Timeout:          120,
	ChannelID:        []string{},
	DefaultUserLimit: 2,
}

type configPayload struct {
	Prefix           string
	Token            string
	Timeout          int
	ChannelID        []string
	DefaultUserLimit int
}

func init() {
	flag.StringVar(&configPath, "c", "server.conf", "Specify configuration file location.")
}

func parse() {
	if _, err := toml.DecodeFile(configPath, &config); err != nil {
		if os.IsNotExist(err) {
			var file *os.File
			if file, err = os.Create(configPath); err != nil {
				log.Fatalf("Error while creating configuration file, %s", err)
				os.Exit(1)
			}
			if err = toml.NewEncoder(file).Encode(defaultConfig); err != nil {
				log.Fatalf("Error while encoding default configuration, %s", err)
				os.Exit(1)
			}
			log.Warnf("Default configuration generated at %s, edit before next startup.", configPath)
			os.Exit(1)
		}
		log.Fatalf("Error while decoding configuration file, %s", err)
		os.Exit(1)
	} else {
		log.Infof("Loaded config at %s.", configPath)
	}
}

func configLate() {
	if config.DefaultUserLimit > 99 || config.DefaultUserLimit < 1 {
		log.Warnf("Value %v is out of range for default user limit.", config.DefaultUserLimit)
		config.DefaultUserLimit = defaultConfig.DefaultUserLimit
	}
	timeout = time.Duration(config.Timeout) * time.Second
	allowedChannels = make(map[string]bool)
	categoryMap = make(map[string]string)
	guildMap = make(map[string]string)
	for _, id := range config.ChannelID {
		if channel, err := session.Channel(id); err != nil {
			log.Errorf("Error getting channel %s, %s", id, err)
			continue
		} else {
			if channel.ParentID == "" {
				log.Warnf("Channel %s has no parent.", id)
				continue
			} else {
				if guildMap[channel.GuildID] != "" {
					log.Warnf("Multiple channels specified for guild %s.", channel.GuildID)
					continue
				} else {
					log.Infof("Channel %s from guild %s with parent %s added.",
						channel.ID, channel.GuildID, channel.ParentID)
					guildMap[channel.GuildID] = channel.ID
					categoryMap[channel.ID] = channel.ParentID
				}
			}
		}
		allowedChannels[id] = true
	}
}
