package main

import (
	"git.randomchars.net/freenitori/log"
	"git.randomchars.net/freenitori/multiplexer"
	"github.com/bwmarrin/discordgo"
)

func init() {
	m.VoiceStateUpdate = append(m.VoiceStateUpdate, handleChatInitiate)
	m.VoiceStateUpdate = append(m.VoiceStateUpdate, handleInstanceChannelActivity)
}

var allowedChannels map[string]bool

func handleChatInitiate(context *multiplexer.Context) {
	// Lookup channel in allowed map
	if context.Channel == nil || !allowedChannels[context.Channel.ID] {
		log.Debug("Chat initiate skipped on irrelevant channel.")
		return
	}

	// Get event
	var event *discordgo.VoiceStateUpdate
	if e, ok := context.Event.(*discordgo.VoiceStateUpdate); !ok {
		log.Debug("Chat initiate skipped on incorrect event.")
		return
	} else {
		event = e
	}
	if event.VoiceState != nil {
		if event.VoiceState.ChannelID == "" {
			log.Debug("Chat initiate skipped on empty channel ID.")
			return
		}
	} else {
		log.Debug("Chat initiate skipped on nil voice state.")
		return
	}

	// Lookup already existing channel
	if instancesUser[event.UserID] != nil {
		log.Debug("Chat initiate skipped on existing channel owner.")
		return
	}

	// Get member from cache and add to cache if not exist
	var member *discordgo.Member
	if u, err := context.Session.State.Member(event.GuildID, event.UserID); err != nil {
		if u, err = context.Session.GuildMember(event.GuildID, event.UserID); err != nil {
			log.Errorf("Error getting member %s (%s), %s", event.UserID, event.GuildID, err)
			return
		} else {
			member = u
			_ = context.Session.State.MemberAdd(u)
		}
	} else {
		member = u
	}
	if member == nil {
		log.Debug("Chat initiate skipped on nil member.")
		return
	}

	// Workaround for weird bug where the event has no guild ID
	if member.GuildID == "" {
		member.GuildID = event.GuildID
	}

	// Create new instance
	instance := newInstance(member)
	if instance == nil {
		log.Debug("Chat initiate skipped on nil instance.")
		return
	}

	log.Infof("%s#%s (%s) has created a new volatile channel #%s (%s) in guild %s (%s).",
		member.User.Username,
		member.User.Discriminator,
		member.User.ID,
		instance.Channel.Name,
		instance.Channel.ID,
		context.Guild.Name,
		context.Guild.ID,
	)

	// Move user to newly created volatile channel
	if err := session.GuildMemberMove(event.GuildID, event.UserID, &instance.Channel.ID); err != nil {
		log.Errorf("Error moving user %s#%s (%s) to newly created volatile channel #%s (%s), %s",
			member.User.Username,
			member.User.Discriminator,
			member.User.ID,
			instance.Channel.Name,
			instance.Channel.ID,
			err)
	}
}

func handleInstanceChannelActivity(context *multiplexer.Context) {
	// Get event
	var event *discordgo.VoiceStateUpdate
	if e, ok := context.Event.(*discordgo.VoiceStateUpdate); !ok {
		return
	} else {
		event = e
	}

	// Lookup channel in pre-event state
	if event.BeforeUpdate == nil || instancesChannel[event.BeforeUpdate.ChannelID] == nil {
		// Add member if join
		instance := instancesChannel[event.ChannelID]
		if instance != nil {
			instance.Members[event.UserID] = true
		}
		return
	}

	// If both before update and current state are same and registered, do not handle
	if event.BeforeUpdate.ChannelID == event.ChannelID && instancesChannel[event.ChannelID] != nil {
		return
	}

	// Remove user from previous channel and destroy if zero length
	instanceOld := instancesChannel[event.BeforeUpdate.ChannelID]
	delete(instanceOld.Members, event.UserID)
	if len(instanceOld.Members) == 0 {
		instanceOld.destroy()
	}

	// Record user in new channel if registered
	instance := instancesChannel[event.ChannelID]
	if instance != nil {
		instance.Members[event.UserID] = true
	}
}
